@@ -7,7 +7,7 @@
 #' @param idmsms filepath: optional idMSMS / MSe csv data.  same dim and names as ms required
 #' @param MStag character: character string in 'taglocation' to designat MS / MSe files e.g. "01.cdf"
 #' @param idMSMStag character: character string in 'taglocation' to designat idMSMS / MSe files e.g. "02.cdf"
-#' @param taglocation character: "filepaths" by default, "phenoData[,1]" is another option. referse to xcms slot
+#' @param taglocation character: "filepaths" by default, "phenoData[,1]" is another option. refers to xcms slot
 #' @param featdelim character: how feature mz and rt are delimited in csv import column header e.g. ="-"
 #' @param timepos integer: which position in delimited column header represents the retention time (csv only)
 #' @param st numeric: sigma t - time similarity decay value 
@@ -19,8 +19,8 @@
 #' @param hmax numeric: precut the tree at this height, default 0.3 - see ?cutreeDynamicTree
 #' @param sampNameCol integer: which column from the csv file contains sample names?
 #' @param collapse logical: reduce feature intensities to spectrum intensities?
-#' @param usePheno logical: tranfer phenotype data from XCMS object to SpecAbund dataset?
-#' @param mspout logical: write msp formatted specta to file?
+#' @param usePheno logical: transfer phenotype data from XCMS object to SpecAbund dataset?
+#' @param mspout logical: write msp formatted spectra to file?
 #' @param ExpDes either an R object created by R ExpDes object: data used for record keeping and labelling msp spectral output
 #' @param normalize character: either "none", "TIC", "quantile", or "batch.qc" normalization of feature intensities.  see batch.qc overview in details. 
 #' @param qc.inj.range integer: how many injections around each injection are to be scanned for presence of QC samples when using batch.qc normalization?  A good rule of thumb is between 1 and 3 times the typical injection span between QC injections.  i.e. if you inject QC ever 7 samples, set this to between 7 and 21.  smaller values provide more local precision but make normalization sensitive to individual poor outliers (though these are first removed using the boxplot function outlier detection), while wider values provide less local precision in normalization but better stability to individual peak areas.
@@ -32,8 +32,8 @@
 #' @param mzdec integer: number of decimal places used in printing m/z values
 #' @param cor.method character: which correlational method used to calculate 'r' - see ?cor
 #' @param rt.only.low.n logical: default = TRUE  At low injection numbers, correlational relationships of peak intensities may be unreliable.  by defualt ramclustR will simply ignore the correlational r value and cluster on retention time alone.  if you wish to use correlation with at n < 5, set this value to FALSE.
-#' @param fftempdir valid path: if there are file size limitations on the default ff pacakge temp directory  - getOptions('fftempdir') - you can change the directory used as the fftempdir with this option.
-#' @param replace.zeros logincal: TRUE by default.  NA, NaN, and Inf values are replaced with zero, and zero values are sometimes returned from peak peaking.  When TRUE, zero values will be replaced with a small amount of noise, with noise level set based on the detected signal intensities for that feature. 
+#' @param fftempdir valid path: if there are file size limitations on the default ff package temp directory  - getOptions('fftempdir') - you can change the directory used as the fftempdir with this option.
+#' @param replace.zeros logical: TRUE by default.  NA, NaN, and Inf values are replaced with zero, and zero values are sometimes returned from peak peaking.  When TRUE, zero values will be replaced with a small amount of noise, with noise level set based on the detected signal intensities for that feature. 
 #' @details Main clustering function output - see citation for algorithm description or vignette('RAMClustR') for a walk through.  batch.qc. normalization requires input of three vectors (1) batch (2) order (3) qc.   This is a feature centric normalization approach which adjusts signal intensities first by comparing batch median intensity of each feature (one feature at a time) QC signal intensity to full dataset median to correct for systematic batch effects and then secondly to apply a local QC median vs global median sample correction to correct for run order effects.
 #' @return   $featclus: integer vector of cluster membership for each feature
 #' @return   $frt: feature retention time, in whatever units were fed in (xcms uses seconds, by default)
@@ -50,11 +50,11 @@
 #' @return   $MSMSdata: the (optional) MSe/idMSMS dataset provided be either xcms or csv input
 #' @return   $SpecAbund: the cluster intensities after collapsing features to clusters
 #' @return   $SpecAbundAve: the cluster intensities after averaging all samples with identical sample names
-#' @return   - 'spectra' directory is created in the working directory.  In this directory a .msp is (optionally) created, which contains the spectra for all compounds in the dataset following clustering.  if MSe/idMSMS data are provided, they are listed witht he same compound name as the MS spectrum, with the collision energy provided in the ExpDes object provided to distinguish low from high CE spectra. 
+#' @return   - 'spectra' directory is created in the working directory.  In this directory a .msp is (optionally) created, which contains the spectra for all compounds in the dataset following clustering.  if MSe/idMSMS data are provided, they are listed width he same compound name as the MS spectrum, with the collision energy provided in the ExpDes object provided to distinguish low from high CE spectra. 
 #' @importFrom grDevices dev.off pdf
 #' @importFrom graphics abline axis boxplot hist "legend" "par" "plot" "points" "title"
 #' @importFrom stats aggregate cor fitted lm loess median predict quantile sd weighted.mean
-#' @importFrom utils edit read.csv read.delim2 write.csv packageVersion
+#' @importFrom utils edit read.csv read.delim read.delim2 write.csv packageVersion
 #' @importFrom ff ff
 #' @importFrom fastcluster hclust
 #' @importFrom dynamicTreeCut cutreeDynamicTree
@@ -232,8 +232,8 @@ ramclustR  <- function(xcmsObj=NULL,
     
     OK <- FALSE
     
-    if(class(xcmsObj)=="xcmsSet") {OK <- TRUE} 
-    if(class(xcmsObj) == "XCMSnExp") {
+    if(inherits(xcmsObj, "xcmsSet")) {OK <- TRUE} 
+    if(inherits(xcmsObj, "XCMSnExp")) {
       OK <- TRUE
       newXCMS <- TRUE
     }
@@ -359,7 +359,7 @@ ramclustR  <- function(xcmsObj=NULL,
   history <- paste(history, "RAMClustR (version ",
                    packageVersion("RAMClustR"),
                    ") was utilized to cluster features into spectra (Broeckling 2014).",
-                   sep = "")  
+                   sep = "")
   
   if(mslev == 2) {
     history <- paste(history,
@@ -654,131 +654,29 @@ ramclustR  <- function(xcmsObj=NULL,
   featnames<-paste(mzs, "_", times, sep="")
   dimnames(data1)[[2]]<-featnames
   dimnames(data2)[[2]]<-featnames
-  
-  ########
-  # establish some constants for downstream processing
   n<-ncol(data1)
-  vlength<-(n*(n-1))/2
-  nblocks<-floor(n/blocksize)
   
   ########
-  # set off ff matrix system for holding data. 
-  # manages RAM demands a bit.  
-  ffmat<-ff::ff(vmode="double", dim=c(n, n), initdata = 0) ##reset to 1 if necessary
-  gc()
-  #Sys.sleep((n^2)/10000000)
-  #gc()
-  
-  ########
-  # make list of all row and column blocks to evaluate
-  eval1<-expand.grid(0:nblocks, 0:nblocks)
-  names(eval1)<-c("j", "k") #j for cols, k for rows
-  eval1<-eval1[which(eval1[,"j"]<=eval1[,"k"]),] #upper triangle only
-  bl<-nrow(eval1)
-  cat(paste("calculating ramclustR similarity: nblocks = ", bl, '\n'))
-  
-  ########
-  # Define the RCsim function used to calculate feature similarities on selected blocks of data
-  RCsim<-function(bl)  {
-    cat(bl,' ')
-    j<-eval1[bl,"j"]  #columns
-    k<-eval1[bl,"k"]  #rows
-    startc<-min((1+(j*blocksize)), n)
-    if ((j+1)*blocksize > n) {
-      stopc<-n} else {
-        stopc<-(j+1)*blocksize}
-    startr<-min((1+(k*blocksize)), n)
-    if ((k+1)*blocksize > n) {
-      stopr<-n} else {
-        stopr<-(k+1)*blocksize}
-    if(startc<=startr) { 
-      mint<-min(abs(outer(range(times[startr:stopr]), range(times[startc:stopc]), FUN="-")))
-      if(mint<=maxt) {
-        temp1<-round(exp(-(( (abs(outer(times[startr:stopr], times[startc:stopc], FUN="-"))))^2)/(2*(st^2))), 
-                     
-                     digits=20 )
-        
-        if(nrow(data1) < 5 & rt.only.low.n) {
-          temp2 <- matrix(data = 1, nrow = length(startr:stopr), ncol = length(startc:stopc))
-        } else {
-          temp2<-round (exp(-((1-(pmax(  cor(data1[,startr:stopr], data1[,startc:stopc], method=cor.method),
-                                         cor(data1[,startr:stopr], data2[,startc:stopc], method=cor.method),
-                                         cor(data2[,startr:stopr], data2[,startc:stopc], method=cor.method)  )))^2)/(2*(sr^2))), 
-                        
-                        digits=20 )	
-        }
-        #ffcor[startr:stopr, startc:stopc]<-temp
-        temp<- 1-(temp1*temp2)
-        temp[which(is.nan(temp))]<-1
-        temp[which(is.na(temp))]<-1
-        temp[which(is.infinite(temp))]<-1
-        ffmat[startr:stopr, startc:stopc]<-temp
-        rm(temp1); rm(temp2); rm(temp)
-        gc()} 
-      if(mint>maxt) {ffmat[startr:stopr, startc:stopc]<- 1}
-    }
-    gc()}
-  
-  ########
-  # Call the similarity scoring function
-  system.time(sapply(1:bl, RCsim))
-  
-  b<-Sys.time()
-  
-  ########
-  # Report progress and timing
-  cat('\n', "RAMClust feature similarity matrix calculated and stored:", '\n')
-  gc() 
-  
-  
-  ########
-  # extract lower diagonal of ffmat as vector
-  blocksize<-mult*round(blocksize^2/n)
-  nblocks<-floor(n/blocksize)
-  remaind<-n-(nblocks*blocksize)
-  
-  ########
-  # create vector for storing dissimilarities
-  ramclustObj<-vector(mode="integer", length=vlength)
-  
-  ########
-  # fill vector with dissimilarities
-  for(k in 0:(nblocks)){
-    startc<-1+(k*blocksize)
-    if ((k+1)*blocksize > n) {
-      stopc<-n} else {
-        stopc<-(k+1)*blocksize}
-    temp<-ffmat[startc:nrow(ffmat),startc:stopc]
-    temp<-temp[which(row(temp)-col(temp)>0)]
-    if(exists("startv")==FALSE) startv<-1
-    stopv<-startv+length(temp)-1
-    ramclustObj[startv:stopv]<-temp
-    gc()
-    startv<-stopv+1
-    rm(temp)
-    gc()
-  }    
-  rm(startv)
-  gc()
+  # calculate similarity matrix
+  ramclustObj <- calculate.similarity(n, data1, data2, times, blocksize, mult, maxt, st, sr, rt.only.low.n, cor.method)
   
   ########
   # convert vector to distance formatted object
-  ramclustObj<-structure(ramclustObj, Size=(n), Diag=FALSE, Upper=FALSE, method="RAMClustR", Labels=featnames, class="dist")
+  ramclustObj <-
+    structure(
+      ramclustObj,
+      Size = (n),
+      Diag = FALSE,
+      Upper = FALSE,
+      method = "RAMClustR",
+      Labels = featnames,
+      class = "dist"
+    )
   gc()
   
-  c<-Sys.time()    
+  c <- Sys.time()
   cat("RAMClust distances converted to distance object", '\n')
   
-  ########
-  # cleanup
-  close(ffmat)
-  rm(ffmat)
-  gc()
-  if(!is.null(fftempdir)) {
-    options("fftempdir" = origffdir)
-  }
-  
-  
   ########
   # cluster using fastcluster package,
   system.time(ramclustObj<-fastcluster::hclust(ramclustObj, method=linkage))
